/**
 * JASMINe
 * Copyright (C) 2010-2011 Bull S.A.S.
 * Copyright (C) 2010 France Telecom
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.monitoring.view {

import com.adobe.cairngorm.control.CairngormEventDispatcher;

import flash.display.DisplayObjectContainer;
import flash.events.MouseEvent;
import flash.events.TimerEvent;
import flash.geom.Point;
import flash.net.FileFilter;
import flash.utils.Dictionary;
import flash.utils.Timer;

import mx.charts.series.AreaSeries;
import mx.charts.series.LineSeries;
import mx.collections.ArrayCollection;
import mx.containers.Canvas;
import mx.containers.HBox;
import mx.containers.TitleWindow;
import mx.containers.VBox;
import mx.controls.*;
import mx.events.*;
import mx.managers.PopUpManager;
import mx.resources.ResourceManager;
import mx.rpc.events.FaultEvent;

import org.ow2.jasmine.monitoring.eos.common.controls.DateTimePicker;
import org.ow2.jasmine.monitoring.eos.common.events.LoadEvent;
import org.ow2.jasmine.monitoring.eos.common.events.PodMenuEvent;
import org.ow2.jasmine.monitoring.eos.common.events.ServerSideExceptionEvent;
import org.ow2.jasmine.monitoring.eos.common.managers.PodLayoutManager;
import org.ow2.jasmine.monitoring.eos.common.view.AreaChartView;
import org.ow2.jasmine.monitoring.eos.common.view.CartesianChartClass;
import org.ow2.jasmine.monitoring.eos.common.view.LineChartView;
import org.ow2.jasmine.monitoring.eos.common.view.LoadWindow;
import org.ow2.jasmine.monitoring.eos.common.view.Notification;
import org.ow2.jasmine.monitoring.eos.common.view.Pod;
import org.ow2.jasmine.monitoring.eos.monitoring.event.*;
import org.ow2.jasmine.monitoring.eos.monitoring.model.*;
import org.ow2.jasmine.monitoring.eos.monitoring.vo.*;
import org.ow2.kerneos.common.event.KerneosNotificationEvent;

/**
 * This is the action script part of the Monitoring EoS module
 * @author durieuxp
 */
public class DashboardClass extends VBox {

    protected const model:MonitoringModelLocator = Monitoring.getInstance().getModel();
    private const dispatcher:CairngormEventDispatcher = Monitoring.getInstance().getDispatcher();

    // dashboard Id.
    protected var did:int = -1;

    protected var podLayoutManager:PodLayoutManager = null;

    // series informations: [serId] -> Serie
    protected var seriesInfo:Dictionary = null;

    // notif filters: [notifName] -> NotifFilter
    protected var notifFilters:Dictionary = null;

    // series data providers: [serId] -> ArrayCollection[date:Date, value:String]
    protected var seriesDataProviders:Dictionary = null;

    /**
     * Current Pod Id in case of menu action
     * this Id is used to index all the following Dictionaries
     */
    protected var podId:String;

    /**
     * List of ChartBase used in this dashboard
     * graphCharts[podId] -> CartesianChartClass
     */
    protected var graphCharts:Dictionary = null;

    /**
     * List of Pods
     *  graphPods[podId] -> Pod
     */
    protected var graphPods:Dictionary = null;

    // for each graph, give the list of series
    // [graph.id] -> Array[serie.id]
    // needed when changing types of graphs
    protected var dataProviderTable:Dictionary = null;

    // for each Graph, give the list of notifFilters
    //  [graph.id] -> Array[notifFilters.name]
    protected var notificationTable:Dictionary = null;

    // type of the graph (area, line)
    protected static const LINE_CHART:int = 0;
    protected static const AREA_CHART:int = 1;

    // Limit dates chosen by the user
    protected var databaseFrom:Date;
    protected var databaseTo:Date;

    protected static const INDEX_GRAPH_IN_POD:int = 0;

    // Config File, with params replaced by their value
    public var strConfigFile:String = "";

    public var replayFileName:String;

    // true if mode replay, false if live monitoring.
    protected var replayMode:Boolean = false;

    // true if replay has been completed.
    protected var replayCompleted:Boolean = false;

    // Last state before reading new data
    protected var lastState:String;

    // timestamp of first data read
    protected var dateFirst:Number = 0;

    // timestamp of last data we can read
    protected var dateLast:Number = 0;

    // timestamp of last data read
    protected var dateCurrent:Number = 0;

    // timestamp of first data currently displayed
    protected var zoomFrom:Number = 0;

    // timestamp of last data currently displayed
    protected var zoomTo:Number = 0;

    /**
     * For each podId, zoomOn is true if zoom has been selected.
     */
    protected var zoomOn:Dictionary = new Dictionary();

    [Bindable]
    protected var modeNames:ArrayCollection = new ArrayCollection([
        locale('stopped'),
        locale('live_monitoring'),
        locale('replay_server_file'),
        locale('replay_local_file'),
        locale('replay_database')
    ]);

    // -------------------------------------------------------------------
    // These variables are defined in the mxml part.
    // -------------------------------------------------------------------

    [Bindable]
    public var filename:Label;

    [Bindable]
    public var headerLine2:HBox;

    [Bindable]
    public var header2:HBox;

    [Bindable]
    public var monitoringMode:ComboBox;

    [Bindable]
    public var prevButton:LinkButton;

    [Bindable]
    public var nextButton:LinkButton;

    [Bindable]
    public var pauseButton:LinkButton;

    [Bindable]
    public var startButton:LinkButton;

    [Bindable]
    public var graphsCanvas:Canvas;

    [Bindable]
    public var dateFrom:DateTimePicker;

    [Bindable]
    public var dateTo:DateTimePicker;

    [Bindable]
    public var applyZoomButton:LinkButton;

    [Bindable]
    public var refreshTimeSlider:HSlider;

    // -------------------------------------------------------------------
    // Initialization
    // -------------------------------------------------------------------

    public function DashboardClass() {
    }

    public function init(strConfigFile:String):void {
        this.strConfigFile = strConfigFile;
    }

    public function initComponent():void {
        debug("init Dashboard")
        // Listen to server side events
        dispatcher.addEventListener(ServerSideExceptionEvent.SERVER_SIDE_EXCEPTION, ServerSideExceptionEvent.show);

        // parse XML configuration
        if (! parseXmlFile()) {
            debug("Error on parsing XML configuration")
            Monitoring.getInstance().mainView.removeTab(this as DashboardView);
            return;
        }

        // Unset the did, because it has been initialized with a bad value from model
        // (actually: the last current value)
        // This is done when this object is built, because of the binding.
        this.did = -3;

        // Ask the Server to create a new dashboard with this list of series.
        var ev:MonitoringEvent = new MonitoringEvent(MonitoringEvent.CREATE_DASHBOARD);
        ev.series = new ArrayCollection();
        for each (var s:Serie in this.seriesInfo) {
            ev.series.addItem(s);
        }
        ev.notifs = new ArrayCollection();
        for each (var n:NotifFilter in this.notifFilters) {
            ev.notifs.addItem(n);
        }
        dispatcher.dispatchEvent(ev);
        
        debug("CREATE_DASHBOARD on server")
    }

    // -------------------------------------------------------------------
    // State Changes
    // -------------------------------------------------------------------

    /**
     * Enter state "stopped"
     * No data is collected, no graph is displayed.
     */
    protected function enterStopped():void {
        header2.setStyle("backgroundColor", "#D0D0D0");

        this.filename.width = 0;
        this.dbwin = null;
        replayCompleted = false;

        // reset all data
        flushProviders();

        startButton.enabled = false;
        pauseButton.enabled = false;
        nextButton.enabled = false;
        prevButton.enabled = false;

        mainMenuData[INDEX_FOLLOW].enabled = false;
    }

    /**
     * Enter state "pause"
     */
    protected function enterPause():void {
        if (replayCompleted) {
            header2.setStyle("backgroundColor", "#E8E8FF");
            startButton.enabled = false;
        } else {
            header2.setStyle("backgroundColor", "#E8E8FF");
            startButton.enabled = true;
        }
        pauseButton.enabled = false;
        resetPrevNextButtons();

        mainMenuData[INDEX_FOLLOW].enabled = true;
    }

    /**
     * Enter state "polling"
     * The timer has been started to periodically get data from the server.
     */
    protected function enterPolling():void {
        header2.setStyle("backgroundColor", "#E0FFE0");

        startButton.enabled = false;
        pauseButton.enabled = true;
        nextButton.enabled = false;
        prevButton.enabled = false;

        if (pollDelayed || ! timerStarted) {
            // Do not wait for timer: poll at once.
            poll(null);
        }
    }

    /**
     * Enter state "waiting"
     * We are waiting for new data from the server.
     */
    protected function enterWaiting():void {
        header2.setStyle("backgroundColor", "#FFA0D0");

        startButton.enabled = false;
        pauseButton.enabled = true;
        nextButton.enabled = false;
        prevButton.enabled = false;
    }

    /**
     * Enter state updating (should comes from waiting)
     */
    protected function enterUpdating():void {
        header2.setStyle("backgroundColor", "#FFD0A0");
    }

    /**
     * The combo box value (mode) has been changed by user.
     */
    protected function modeChange():void {
        var newMode:String = monitoringMode.text; // save it before

        // mode has changed: Force a reset before.
        if (currentState != "stopped") {
            stopLiveMonitoring();
        }
        if (followData) {
            unsetFollowData();
            followData = false;
            mainMenuData[INDEX_FOLLOW].toggled = false;
        }
        currentState = "stopped";

        if (newMode == locale('live_monitoring')) {
            // switch mode to "live monitoring"
            startLiveMonitoring();
        } else if (newMode == locale('replay_server_file')) {
            // switch mode to "replay from server file"
            var rwin:ReplayWindow = PopUpManager.createPopUp(this.graphsCanvas, ReplayWindow, true) as ReplayWindow;
            rwin.addEventListener(ReplayEvent.FILE_CHOSEN, replayFile);
            PopUpManager.centerPopUp(rwin);
        } else if (newMode == locale('replay_local_file')) {
            // switch mode to "replay from local file"
            var lwin:LoadWindow = PopUpManager.createPopUp(this.graphsCanvas, LoadWindow, true) as LoadWindow;
            var xmlTypes:FileFilter = new FileFilter("Log files (*.log, *.txt, *csv)", "*.log;*.txt;*.csv");
            lwin.fileFilter = new Array(xmlTypes);
            lwin.title = locale('load_log_file');
            lwin.addEventListener(LoadEvent.LOADED, replayLocalFile);
            PopUpManager.centerPopUp(lwin);
        } else if (newMode == locale('replay_database')) {
            // switch mode to "replay from database"
            dbwin = PopUpManager.createPopUp(this.graphsCanvas, DatabaseWindow, true) as DatabaseWindow;
            dbwin.init(this);
            getDatabaseLimits();
            PopUpManager.centerPopUp(dbwin);
        }
    }

    // -------------------------------------------------------------------
    // Main Menu (Actions)
    // -------------------------------------------------------------------

    protected var mainMenu:Menu = null;

    /**
     * Open the menu
     */
    protected function mainMenuOpen(e:MouseEvent):void {
        mainMenu = Menu.createMenu(null, mainMenuData, false);
        mainMenu.addEventListener(MenuEvent.ITEM_CLICK, mainMenuClick);

        // get the menu parent from the mouse event
        var target:DisplayObjectContainer = DisplayObjectContainer(e.currentTarget);

        // position menu relative to the parent
        var point:Point = new Point(e.localX, e.localY);
        point = target.localToGlobal(point);

        // show the menu just below the link button
        mainMenu.show(point.x - 5, point.y + 16);
    }

    /**
     * Close the Menu
     */
    protected function mainMenuClose(e:MouseEvent):void {
        mainMenu.hide();
        mainMenu = null;
    }

    private const INDEX_FOLLOW:int = 0;

    protected var mainMenuData:Array = [
        {label: locale("follow_data"), type: "check", id: "follow", toggled: false},
        {label: locale("rename_dashboard"), type: "normal", id: "rename"},
    ];

    public function toggleMainMenu(e:MouseEvent):void {
        if (mainMenu != null) {
            mainMenuClose(e);
        }
        else {
            mainMenuOpen(e);
        }
    }

    /**
     * Main Menu has been used
     * @param menuEvent
     */
    protected function mainMenuClick(menuEvent:MenuEvent):void {
        mainMenu = null;
        var id:String = menuEvent.item.id;
        if (id == "follow") {
            followData = menuEvent.item.toggled;
            if (followData) {
                setFollowData();
            } else {
                unsetFollowData();
            }
        }
        else if (id == "rename") {
            // rename dashboard
            var gnwin:DashboardOptions = PopUpManager.createPopUp(this.graphsCanvas, DashboardOptions, true) as DashboardOptions;
            gnwin.title = locale("dashboard_option_setting");
            gnwin.init(this);
            PopUpManager.centerPopUp(gnwin);
        }
        else {
            Alert.show("Menu Item Clicked: " + menuEvent.item.label, "Menu Click");
        }
    }

    protected function closeWindow():void {
        stopLiveMonitoring();
        currentState = "stopped";

        // Notify server that this dashboard is removed
        removeMe();
    }

    // -------------------------------------------------------------------
    // Pod Menu (options)
    // -------------------------------------------------------------------

    protected var podmenu:Menu = null;

    protected var podMenuData:Array = [
        {label: locale("export_graph"), type: "normal", id: "export"},
        {label: locale("zoom"), type: "check", id: "zoom", toggled: false},
        {label: locale("graph_type"), id: "type", children: [
            {label: locale("line"), id: "line", type: "radio", toggled: false},
            {label: locale("area"), id: "area", type: "radio", toggled: false}
        ]}
    ];

    /**
     * Pod Menu has been clicked
     * @param e
     */
    protected function podMenuOpen(e:PodMenuEvent):void {
        // Keep the current podId for later use
        podId = e.podId;

        // Init the toggled elements in the menu
        // This is needed because the data menu is shared by all the pods.
        for (var i:int = 0; i < podMenuData.length; i++) {
            var elem:Object = podMenuData[i];
            if (elem.id == "zoom") {
                elem.toggled = zoomOn[podId];
            }
            else if (elem.id == "type") {
                for (var j:int = 0; j < elem.children.length; j++) {
                    var elem2:Object = elem.children[j];
                    if (elem2.id == "line") {
                        elem2.toggled = (graphCharts[podId] is LineChartView);
                    }
                    else if (elem2.id == "area") {
                        elem2.toggled = (graphCharts[podId] is AreaChartView);
                    }
                }
            }
        }

        podmenu = Menu.createMenu(null, podMenuData, false);
        podmenu.addEventListener(MenuEvent.ITEM_CLICK, podMenuClick);

        // get the menu parent from the mouse event
        var target:DisplayObjectContainer = DisplayObjectContainer(e.mouseEvent.currentTarget);

        // position menu relative to the parent
        var point:Point = new Point(e.mouseEvent.localX, e.mouseEvent.localY);
        point = target.localToGlobal(point);

        // show the menu just below the link button
        podmenu.show(point.x - 50, point.y + 16);
    }

    protected function podMenuClose(e:PodMenuEvent):void {
        podmenu.hide();
        podmenu = null;
    }

    public function togglePodMenu(e:PodMenuEvent):void {
        if (podmenu != null) {
            podMenuClose(e);
        }
        else {
            podMenuOpen(e);
        }
    }

    /**
     * Pod Menu has been used
     * @param menuEvent
     */
    protected function podMenuClick(menuEvent:MenuEvent):void {
        podmenu = null;
        var id:String = menuEvent.item.id;
        if (id == "export") {
            graphCharts[podId].export();
        }
        else if (id == "zoom") {
            graphCharts[podId].zoom(menuEvent.item.toggled);
            zoomOn[podId] = menuEvent.item.toggled;
        }
        else if (id == "line") {
            changeGraphType(LINE_CHART);
        }
        else if (id == "area") {
            changeGraphType(AREA_CHART);
        }
        else {
            Alert.show("Menu Item Clicked: " + menuEvent.item.label, "Menu Click");
        }
    }

    // -------------------------------------------------------------------
    // Setters triggered by bindings
    // -------------------------------------------------------------------

    /**
     * New dashboard created: Store its identifier, chosen by the server.
     * @param event
     */
    public function set dashboard(id:int):void {
        if (this.did < 0 && id > 0) {
            // This dashboard has been succesfully created on server.
            this.did = id;
            this.currentState = "stopped";
        }
    }

    /**
     * Error from Server
     */
    public function set error(event:FaultEvent):void {
        Alert.show(locale("server_disconnected"), locale("error_occured"));
        stopTimer();
    }

    public var dbwin:DatabaseWindow = null;

    public function set timePickers(timestamp:Number):void {
        if (timestamp > 0) {
            dateFirst = timestamp;
            dateLast = (new Date()).getTime() + 1800000; // 30 mn later
            zoomFrom = dateFirst;
            zoomTo = dateLast;
            if (dbwin != null) {
                dbwin.dateFromDB.selectedDate = new Date(zoomFrom);
                dbwin.dateToDB.selectedDate = new Date(zoomTo);
                var first:Date = new Date(dateFirst);
                var last:Date = new Date(dateLast);
                dbwin.dateFromDB.setBoundaries(first,  last);
                dbwin.dateToDB.setBoundaries(first,  last);
            }
        }
    }

    /**
     * Fill in the providers with datas.
     * TODO Check if data not sent many times ?
     * @param prov Collection of EventData
     */
    public function set liveProviders(prov:ArrayCollection):void {
        // Check dashboard Id
        if (model.dbIdent != did) {
            // These data are not for me.
            if (model.dbIdent > 0) {
                // Alert.show("liveProviders Data not for me: "+ did + " != " + model.dbIdent);
            }
            return;
        }

        // If end of replay : stop the timer if it is on.
        if (this.replayMode && prov == null) {
            stopTimer();
            replayCompleted = true;
            // set limit to actual limit, since there is no more data to read
            dateLast = zoomTo;
            this.currentState = "pause";
            return;
        }

        // If not waiting data, forget them
        if (this.currentState != "waiting") {
            // Alert.show("forget data not waited");
            return;
        }

        // send data to graphs
        this.currentState = "updating";
        liveDataLoad(prov);
        this.currentState = this.lastState;
    }

    /**
     * Take in account new notifications
     * TODO Check we received them only once
     * @param notifs
     */
    public function set notifications(notifs:ArrayCollection):void {
        debug("set notifications");
        // Check dashboard Id
        if (model.dbIdent != did) {
            // These data are not for me.
            // if (model.dbIdent > 0) {
            //     Alert.show("notifications: Data not for me: "+ did + " != " + model.dbIdent);
            // }
            return;
        }

        // update notifications on each Chart
        for each (var notif:JasmineEventNotificationVO in notifs) {
            var mynotif:Notification = new Notification();
            mynotif.time = notif.date.getTime();
            mynotif.message = notif.message;
            mynotif.level = notif.levelValue; // assumes values are the same !
            // Add notification to each Graph that accept it

            for each (var pod:Pod in graphPods) {
                // If at least one filter accept it, give it.
                for each (var filterName:String in notificationTable[pod.id]) {
                    if (notifFilters[filterName].matches(notif)) {
                        debug("received notif " + notif.message + " kept for " + pod.id);
                        graphCharts[pod.id].addNotification(mynotif);
                        break;
                    }
                }
            }
        }
    }

    // -------------------------------------------------------------------
    // Modes of execution
    // -------------------------------------------------------------------

    /**
     * START REPLAY from Database
     */
    protected function startDatabaseReplay():void {
        var ev:MonitoringEvent = new MonitoringEvent(MonitoringEvent.START_DATABASE_REPLAY);
        ev.ident = this.did;
        ev.from = this.databaseFrom;
        ev.to = this.databaseTo;
        dispatcher.dispatchEvent(ev);
        debug("START_DATABASE_REPLAY on server")

        // Init time range to display
        this.dateFirst = this.databaseFrom.getTime();
        this.dateLast = this.databaseTo.getTime();
        this.zoomFrom = this.dateFirst;
        this.zoomTo = this.zoomFrom + 600000; // 10 mn later
        if (zoomTo > dateLast) {
            zoomTo = dateLast;
        }
        updateDatePickers();

        // Set currentstate
        this.currentState = "pause";

    }

    public function getDatabaseLimits():void {
        if (model.databaseStartTime > 0) {
            // already there
            timePickers = model.databaseStartTime;
        } else {
            // fetch it on the server.
            var ev:MonitoringEvent;
            ev = new MonitoringEvent(MonitoringEvent.GET_DATABASE_LIMITS);
            ev.ident = did;
            dispatcher.dispatchEvent(ev);
        }
    }

    /**
     * START REPLAY from File
     */
    protected function startFileReplay():void {
        var ev:MonitoringEvent = new MonitoringEvent(MonitoringEvent.START_FILE_REPLAY);
        ev.ident = this.did;
        ev.loadFile = this.replayFileName;
        ev.from = this.databaseFrom;
        ev.to = this.databaseTo;
        dispatcher.dispatchEvent(ev);
        debug("START_FILE_REPLAY on server")

        // Init time range to display
        this.dateFirst = this.databaseFrom.getTime();
        this.dateLast = this.databaseTo.getTime();
        this.zoomFrom = this.dateFirst;
        this.zoomTo = this.zoomFrom + 600000; // 10 mn later
        if (zoomTo > dateLast) {
            zoomTo = dateLast;
        }
        updateDatePickers();

        this.currentState = "pause";
    }

    /**
     * START Live Monitoring
     */
    protected function startLiveMonitoring():void {
        this.replayMode = false;
        this.filename.width = 0;

        // Init time range to display
        this.dateFirst = new Date().getTime();
        this.zoomFrom = this.dateFirst;
        this.zoomTo = this.zoomFrom + 600000; // 10 mn later
        this.dateLast = this.zoomTo + 1200000; // later than zoomTo
        updateDatePickers();

        this.currentState = "pause";

        // Start live Monitoring on server
        if (did < 0) {
            Alert.show("Error: Bad dashboard ID " + did);
            return;
        }
        var event:MonitoringEvent = new MonitoringEvent(MonitoringEvent.START_MONITORING);
        event.ident = did;
        dispatcher.dispatchEvent(event);
        debug("START_MONITORING on server")
    }

    protected function stopLiveMonitoring():void {
        // First of all, stop polling data, in case the Timer is on.
        if (this.timerStarted) {
            stopTimer();
        }

        // Stop all activity on server for this dashboard
        if (did > 0) {
            var event:MonitoringEvent = new MonitoringEvent(MonitoringEvent.STOP_MONITORING);
            event.ident = did;
            dispatcher.dispatchEvent(event);
            debug("STOP_MONITORING on server")
        }
    }

    /**
     * A local file has been loaded. We can process the replay.
     * @param event Name of the file to replay
     */
    protected function replayLocalFile(event:LoadEvent):void {
        var rwin:ReplayWindow = PopUpManager.createPopUp(this.graphsCanvas, ReplayWindow, true) as ReplayWindow;
        rwin.addEventListener(ReplayEvent.FILE_CHOSEN, replayFile);
        rwin.preselectFile(event.filename);
        PopUpManager.centerPopUp(rwin);
    }

    /**
     * Replay from file: The File has been previously selected by user.
     * @param event
     */
    protected function replayFile(event:ReplayEvent):void {
        this.replayFileName = event.replayFile;

        // Display only the last part of the full path
        var maxsize:int = 32;
        var index:int = this.replayFileName.length - maxsize;
        var troncated:String = this.replayFileName;
        if (index > 0)
            troncated = this.replayFileName.substring(index);
        this.filename.text = troncated;
        this.filename.width = 120;

        this.replayMode = true;
        this.databaseFrom = new Date(event.from);
        this.databaseTo = new Date(event.to);
        this.timerLength = 2; // default value for replay, can be changed by user.
        startFileReplay();
    }

    /**
     * Replay from Database.
     * This is called when getting out of the DatabaseWindow
     * @param from
     * @param to
     */
    public function replayDatabase(from:Date, to:Date):void {
        this.replayMode = true;
        this.databaseFrom = from;
        this.databaseTo = to;
        this.timerLength = 2; // default value for replay, can be changed by user.
        startDatabaseReplay();
    }

    /**
     * Pod has been created so update the respective PodLayoutManager.
     * FIXME: Should wait that ALL pods have been completed their creation.
     */
    protected function displayPods(e:FlexEvent):void {
        e.currentTarget.removeEventListener(FlexEvent.UPDATE_COMPLETE, displayPods);

        // this is really needed ! (why ?)
        this.podLayoutManager.removeNullItems();

        this.podLayoutManager.updateLayout(false);
        // ??? this.podLayoutManager.dispatchEvent(new LayoutChangeEvent(LayoutChangeEvent.UPDATE));
    }

    // -------------------------------------------------------------------
    // Buttons
    // -------------------------------------------------------------------

    /**
     * PREV
     */
    public function prevButtonClick():void {
        prevPage();
    }

    /**
     * PAUSE
     */
    public function pauseButtonClick():void {
        stopTimer();
        this.currentState = "pause";
    }

    /**
     * NEXT
     */
    public function nextButtonClick():void {
            nextPage();
    }

    /**
     * START button has been clicked
     */
    public function startButtonClick():void {
        // Set currentstate
        this.currentState = "polling";

        // create a timer to periodically poll new results
        startTimer();
    }

    /**
     * HELP BUTTON
     */
    public function showHelp():void {
        var helpWindow:MonitoringHelp = PopUpManager.createPopUp(this.graphsCanvas, MonitoringHelp, true) as MonitoringHelp;
        PopUpManager.centerPopUp(helpWindow);
    }

    /**
     * TEXT INPUT: new name of the Tab
     */
    public function changeDashboardName(newname:String):void {
        this.label = newname;
    }

    // -------------------------------------------------------------------
    // Date Picker
    // -------------------------------------------------------------------

    public function fromChanged():void {
        zoomFrom = this.dateFrom.selectedDate.getTime();
    }

    public function toChanged():void {
        zoomTo = this.dateTo.selectedDate.getTime();
    }

    public function refresh():void {
        updateTimeLimits(zoomFrom, zoomTo);
        resetPrevNextButtons();
    }

    // -------------------------------------------------------------------
    // followData
    // -------------------------------------------------------------------

    [Bindable]
    protected var followData:Boolean = false;

    public function unsetFollowData():void {
        this.headerLine2.height = 0;

        // Reset graphs on all datas.
        for each (var ccc:CartesianChartClass in this.graphCharts) {
            ccc.resetTimeLimits();
        }
        if (dateCurrent < dateFirst) {
            Alert.show("error unsetFollowData")
            return;
        }
        zoomFrom = dateFirst;
        zoomTo = dateCurrent;

        updateDatePickers();
        resetPrevNextButtons();
    }

    public function setFollowData():void {
        this.headerLine2.height = 30;
        if (currentState == "pause") {
            resetPrevNextButtons();
        }
    }

    // -----------------------------------------------------------------------------
    // private Methods
    // -----------------------------------------------------------------------------

    protected function resetPrevNextButtons():void {
        if (followData && zoomFrom > dateFirst) {
            prevButton.enabled = true;
        } else {
            prevButton.enabled = false;
        }
        // In liveMonitoring, we can always get new data.
        if (replayMode && zoomTo >= dateLast) {
            nextButton.enabled = false;
        } else {
            nextButton.enabled = true;
        }
    }

    /**
     * Update time limits for all Pods
     * @param from
     * @param to
     */
    protected function updateTimeLimits(from:Number, to:Number):void {
        if (to < from) {
            Alert.show("error updateTimeLimits");
            return;
        }
        for each (var ccc:CartesianChartClass in this.graphCharts) {
            ccc.setTimeLimits(from, to);
        }
    }

    /**
     * MENU change type of graph for current podId
     * @param type new type
     */
    protected function changeGraphType(type:int):void {

        // create a new type of chart with the same list of series
        var seriesList:ArrayCollection = new ArrayCollection();
        var serid:String = null;

        var ccc:CartesianChartClass = null;
        switch (type) {
            case LINE_CHART:
                ccc = new LineChartView();
                for each (serid in this.dataProviderTable[podId]) {
                    var lineSerie:LineSeries = new LineSeries();
                    lineSerie.id = serid;
                    lineSerie.dataProvider = this.seriesDataProviders[serid];
                    lineSerie.displayName = this.seriesInfo[serid].description;
                    lineSerie.yField = "value";
                    lineSerie.xField = "date";
                    seriesList.addItem(lineSerie);
                }
                break;
            case AREA_CHART:
                ccc = new AreaChartView();
                for each (serid in this.dataProviderTable[podId]) {
                    var areaSerie:AreaSeries = new AreaSeries();
                    areaSerie.id = serid;
                    areaSerie.dataProvider = this.seriesDataProviders[serid];
                    areaSerie.displayName = this.seriesInfo[serid].description;
                    areaSerie.yField = "value";
                    areaSerie.xField = "date";
                    seriesList.addItem(areaSerie);
                }
                break;
        }
        ccc.setCharts(seriesList);
        ccc.setDynamicRedraw(false);
        this.graphCharts[podId] = ccc;

        // zoom will be reset when redrawing the pod
        this.zoomOn[podId] = false;

        // replace the graph inside the Pod and refresh LayoutManager
        var pod:Pod = this.graphPods[podId];
        pod.removeChildAt(INDEX_GRAPH_IN_POD);
        pod.addChildAt(ccc, INDEX_GRAPH_IN_POD);
        pod.addEventListener(FlexEvent.UPDATE_COMPLETE, displayPods);

        // redraw graphs
        updateTimeLimits(zoomFrom, zoomTo);
    }

    /**
     * Parse the XML Configuration Data (strConfigFile)
     * Build all VO holding the series and the graphs to draw.
     */
    protected function parseXmlFile():Boolean {
        // Parse the XML file
        var doc:XML = new XML(this.strConfigFile);

        this.graphsCanvas.removeAllChildren();

        // flush Lists
        this.seriesInfo = new Dictionary();
        this.notifFilters = new Dictionary();
        this.seriesDataProviders = new Dictionary();
        this.graphCharts = new Dictionary();
        this.graphPods = new Dictionary();
        this.dataProviderTable = new Dictionary();
        this.notificationTable = new Dictionary();

        this.initPodLayoutManager();

        // for each serie-def
        for each (var node:XML in doc.child("serie-def")) {
            // Build a Serie VO and keep it in seriesInfo
            var serie:Serie = new Serie();
            serie.id = node.@id.toString();
            serie.name = node.name;
            serie.description = node.description;
            if (serie.description == "")
                serie.description = serie.name;
            serie.ycol = node.y.@col;
            serie.ytype = node.y.@type;
            serie.pattern = node.pattern;
            serie.server = node.server;
            this.seriesInfo[serie.id] = serie;

            // init data provider of the serie
            this.seriesDataProviders[serie.id] = new ArrayCollection();
        }

        // for each notification-filter
        for each (var nodeN:XML in doc.child("notification-filter")) {
            // Build a NotifFilter and keep it in notifFilters
            var notif:NotifFilter = new NotifFilter();
            notif.id = nodeN.@id.toString();
            if (nodeN.rule != null) {
                notif.rulePattern = nodeN.rule.@pattern;
            }
            if (nodeN.level != null) {
                notif.setLevel(nodeN.level.@value);
                notif.setLevelPolicy(nodeN.level.@policy);
            }
            // details
            for each (var nodeD:XML in nodeN.child("detail")) {
                notif.setDetail(nodeD.@name, nodeD.@pattern);
            }
            debug("New notif " + notif.id)
            this.notifFilters[notif.id] = notif;
        }

        // for each graph
        var pod:Pod = null;
        for each (var nodeG:XML in doc.child("graph")) {
            var graphId:String = nodeG.@id.toString();

            // title
            var titleValue:String = nodeG.title;

            //type of chart
            var chartType:String = nodeG.chartType;
            var chartSelected:int;
            if (chartType == "lineChartConst" || chartType == "") {
                chartSelected = LINE_CHART;
            } else if (chartType == "areaChartConst") {
                chartSelected = AREA_CHART;
            } else {
                Alert.show(locale("bad_type") + chartType + ". " + "Should be lineChartConst or areaChartConst")
                return false;
            }

            // add notif list for the current graph
            [ArrayElementType("String")]
            var notifList:ArrayCollection = new ArrayCollection();
            for each (var notifNode:XML in nodeG.child("filter")) {
                var notifName:String = notifNode.@ref.toString();
                
                // Check if Notif has been defined
                if (this.notifFilters[notifName] == null) {
                    Alert.show("missing definition notif " + notifName);
                    return false;
                }

                // take in account notifications for the graph
                notifList.addItem(notifName);
                debug("Adding notif " + notifName + " for " + graphId)
            }
            this.notificationTable[graphId] = notifList;
            
            //create an array to add all the series for the current graph
            var graphSeries:Array = new Array;

            // add each serie for the current graph
            var seriesList:ArrayCollection = new ArrayCollection();
            for each (var serieNode:XML in nodeG.child("serie")) {
                var serieId:String = serieNode.@ref.toString();

                // Check if Serie has been defined
                if (this.seriesInfo[serieId] == null) {
                    Alert.show(locale("missing_definition_serie") + serieId);
                    return false;
                }
                graphSeries.push(serieId);

                switch (chartSelected) {
                    case LINE_CHART:
                        var lineSerie:LineSeries = new LineSeries();
                        lineSerie.id = serieId;
                        lineSerie.dataProvider = this.seriesDataProviders[serieId];
                        if (serieNode.toString() != "") {
                            lineSerie.displayName = serieNode.toString();
                        }
                        else {
                            lineSerie.displayName = this.seriesInfo[serieId].description;
                        }
                        lineSerie.yField = "value";
                        lineSerie.xField = "date";
                        seriesList.addItem(lineSerie);
                        break;

                    case AREA_CHART:
                        var areaSerie:AreaSeries = new AreaSeries();
                        areaSerie.id = serieId;
                        areaSerie.dataProvider = this.seriesDataProviders[serieId];
                        if (serieNode.toString() != "") {
                            areaSerie.displayName = serieNode.toString();
                        }
                        else {
                            areaSerie.displayName = this.seriesInfo[serieId].description;
                        }
                        areaSerie.yField = "value";
                        areaSerie.xField = "date";
                        seriesList.addItem(areaSerie);
                        break;
                }

            }
            this.dataProviderTable[graphId] = graphSeries;

            // create the Chart
            var ccc:CartesianChartClass = null;
            switch (chartSelected) {
                case LINE_CHART:
                    ccc = new LineChartView();
                    break;
                case AREA_CHART:
                    ccc = new AreaChartView();
                    break;
            }
            ccc.setCharts(seriesList);
            ccc.setDynamicRedraw(false);
            this.graphCharts[graphId] = ccc;
            this.zoomOn[graphId] = false;

            // Create the pod
            pod = new Pod();
            pod.title = titleValue;
            pod.id = graphId;
            pod.addChildAt(ccc, INDEX_GRAPH_IN_POD);
            pod.addEventListener(PodMenuEvent.MENU, togglePodMenu);

            // Add Pod to the podLayoutManager
            this.podLayoutManager.addItem(pod);

            //filling the table for the correspondence between the pods and
            //the graphs
            this.graphPods[pod.id] = pod;
        }
        if (pod == null) {
            warning("Error parsing xml: pod is null")
            return false;
        }
        // display all the pods just created
        // We must wait that the last created Pod is completed.
        pod.addEventListener(FlexEvent.UPDATE_COMPLETE, displayPods);
        return true;
    }

    /**
     * process the new data received
     * @param provider ArrayCollection of EventDataVO
     */
    protected function liveDataLoad(provider:ArrayCollection):void {

        for each (var evt:EventDataVO in provider) {

            // update dateCurrent and dateLast if needed.
            var time:Number = evt.timestamp;
            if (time > dateCurrent) {
                dateCurrent = time;
                if (dateCurrent > dateLast) {
                    dateLast = dateCurrent;
                }
            }

            if (followData) {
                // Display last data received, in the range specified
                if (time > zoomTo) {
                    var delta:Number = time - zoomTo;
                    zoomFrom = zoomFrom + delta;
                    zoomTo = time;
                }
            } else {
                // update zoomTo if needed
                if (dateCurrent > zoomTo) {
                    zoomTo = dateCurrent;
                    if (zoomTo < zoomFrom) {
                        Alert.show("error liveDataLoad : " + zoomTo + " < " + zoomFrom)
                        return;
                    }
                }
            }
            // Add item to the series that wait for it.
            var item:Object = {date: new Date(time), value: evt.value};
            (this.seriesDataProviders[evt.serid] as ArrayCollection).addItem(item);
        }
        // redraw the graphs
        debug("liveDataLoad from " + zoomFrom + " to " + zoomTo);
        updateTimeLimits(zoomFrom, zoomTo);

        // make DataPicker's reflect the new time range.
        updateDatePickers();
    }

    /**
     * Update date pickers
     */
    protected function updateDatePickers():void {
        dateFrom.selectedDate = new Date(zoomFrom);
        dateTo.selectedDate = new Date(zoomTo);
        var first:Date = new Date(dateFirst);
        var last:Date = new Date(dateLast);
        dateFrom.setBoundaries(first,  last);
        dateTo.setBoundaries(first,  last);
    }

    /**
     * Flush providers for this dashboard.
     */
    protected function flushProviders():void {
        for each (var provider:ArrayCollection in this.seriesDataProviders) {
            // provider.removeAll() doesn't work very well and is too costly
            provider.source = new Array();
        }
        // reset all to avoid side effects
        this.dateCurrent = 0;
        this.dateFirst = 0;
        this.dateLast = 0;
    }

    /**
     * init the podLayoutManager
     */
    protected function initPodLayoutManager():void {
        this.podLayoutManager = new PodLayoutManager();
        this.podLayoutManager.container = this.graphsCanvas;
        this.podLayoutManager.id = "unique";
    }

    /**
     * Remove this dashboard
     */
    protected function removeMe():void {
        var ev:MonitoringEvent = new MonitoringEvent(MonitoringEvent.REMOVE_DASHBOARD);
        ev.ident = this.did;
        dispatcher.dispatchEvent(ev);
        debug("REMOVE_DASHBOARD on server")
        // Remove Tab
        Monitoring.getInstance().mainView.removeTab(this as DashboardView);
    }

    /**
     * SLICER: Set the timer refresh period
     */
    protected function timerRefreshPeriodChange():void {
        setTimerPeriod(this.refreshTimeSlider.value);
    }

    protected function nextPage():void {
        // Find the new time range to display
        var delta:Number = zoomTo - zoomFrom;
        if (delta < 0)  {
            Alert.show("error nextPage")
            return;
        }
        if (replayCompleted && zoomTo + delta > dateCurrent) {
            // Do not try to go further if all has been read already
            delta = dateCurrent - zoomTo;
        }
        if (zoomTo + delta > dateCurrent) {
            // We need more results
            var ev:MonitoringEvent = new MonitoringEvent(MonitoringEvent.GET_LIVE_RESULT);
            ev.ident = did;
            dispatcher.dispatchEvent(ev);
            debug("GET_LIVE_RESULT on server");

            //  Get last notifications too
            var ev2:MonitoringEvent = new MonitoringEvent(MonitoringEvent.GET_NOTIFICATIONS);
            ev2.ident = did;
            dispatcher.dispatchEvent(ev2);
            debug("GET_NOTIFICATIONS on server");

            this.lastState = this.currentState;
            this.currentState = "waiting";
        } else {
            debug("next local zoomTo=" + zoomTo)
            // we already got enough data to display
            zoomFrom = zoomFrom + delta;
            zoomTo = zoomTo + delta;
            refresh();
            updateDatePickers();
        }
    }

    protected function prevPage():void {
        this.lastState = this.currentState;
        // Find the new time range to display
        var delta:Number = zoomTo - zoomFrom;
        if (delta < 0)  {
            Alert.show("error prevPage")
            return;
        }
        zoomFrom = zoomFrom - delta;
        if (zoomFrom < dateFirst) {
            zoomFrom = dateFirst;
        }
        zoomTo = zoomFrom + delta;
        refresh();
        updateDatePickers();
    }

    // -----------------------------------------------------------------------------
    // Monitoring timer
    // -----------------------------------------------------------------------------

    protected var timerStarted:Boolean = false;

    protected var pollDelayed:Boolean = false;

    protected var monitoringTimer:Timer = null;

    // Current period for timer, in seconds.
    [Bindable]
    protected var timerLength:Number = 8;

    /**
     *  create a timer to periodically poll new results
     */
    protected function startTimer():void {
        var period:Number = this.timerLength * 1000;
        if (this.monitoringTimer == null) {
            this.monitoringTimer = new Timer(period, Infinity);
            this.monitoringTimer.addEventListener(TimerEvent.TIMER, poll);
        }
        this.monitoringTimer.start();
        // show current timer period on slicer
        this.refreshTimeSlider.value = this.timerLength;
        this.timerStarted = true;
    }

    /**
     * Callback function for Timer: must have this signature.
     * @param event
     */
    protected function poll(event:TimerEvent):void {
        if (currentState == "waiting" || currentState == "updating") {
            // We have not finished eating last results; do not poll now
            pollDelayed = true;
        } else {
            // currentstate should be "polling" here.
            nextPage();
            pollDelayed = false;
        }
    }

    /**
     * remove the Timer
     */
    protected function stopTimer():void {
        if (this.monitoringTimer != null) {
            this.monitoringTimer.stop();
            this.monitoringTimer = null;
        }
        this.timerStarted = false;
    }

    /**
     * Set the Timer period
     */
    protected function setTimerPeriod(value:int):void {
        this.timerLength = value;

        // If timer already running, change its period accordingly
        if (this.monitoringTimer != null) {
            this.monitoringTimer.delay = this.timerLength * 1000;
        }
    }

    // -----------------------------------------------------------------------------
    // Progress Bar
    // -----------------------------------------------------------------------------

    /**
     * Title window containing the progress Bar.
     */
    protected var titleWindow:TitleWindow = null;
    protected var progressBar:ProgressBar = null;

    /**
     * Show a progress bar.
     */
    protected function showProgressBar(text:String):void {
        hideProgressBar();

        // create the progress bar.
        progressBar = new ProgressBar();
        progressBar.indeterminate = true;
        progressBar.percentWidth = 100;
        progressBar.label = text;

        // create the popup.
        titleWindow = PopUpManager.createPopUp(this.graphsCanvas, TitleWindow, true) as TitleWindow;
        titleWindow.title = locale("wait");
        titleWindow.width = this.width / 4;
        // add the progress bar to the popup
        titleWindow.addChild(progressBar);
        // center the popup.
        PopUpManager.centerPopUp(titleWindow);
    }

    /**
     * Hide the progress Bar.
     */
    protected function hideProgressBar():void {
        if (titleWindow != null) {
            // remove the popup from the screen.
            PopUpManager.removePopUp(titleWindow);
            titleWindow = null;
            progressBar = null;
        }
    }

    // -----------------------------------------------------------------------------
    // Internationalization
    // -----------------------------------------------------------------------------

    /**
     * This is needed only for localization of the label.
     * @return
     */
    protected function getSliderLabels():Array {
        var str_array:Array = new Array();
        str_array[0] = "Os";
        str_array[1] = locale('refresh_period');
        str_array[2] = "60s";
        return  str_array;
    }

    /**
     * Get localized String
     */
    public function locale(s:String):String {
        return ResourceManager.getInstance().getString("monitoringResources", s);
    }
         
    public function debug(message:String):void {
        dispatchEvent(new KerneosNotificationEvent(KerneosNotificationEvent.KERNEOS_NOTIFICATION,
                message, KerneosNotificationEvent.DEBUG));
    }
    public function warning(message:String):void {
        dispatchEvent(new KerneosNotificationEvent(KerneosNotificationEvent.KERNEOS_NOTIFICATION,
                message, KerneosNotificationEvent.WARNING));
    }

}
}
