/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Serie.as 8777 2011-08-31 14:22:51Z diorcety $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.monitoring.vo {
import com.adobe.cairngorm.vo.IValueObject;

import flash.utils.Dictionary;

import mx.controls.Alert;


/**
 * Defines a filter to select which Notifications we are interested in.
 * @author durieuxp
 */
[Bindable]
public class NotifFilter implements IValueObject {

    // Each filter is identified by an id
    private var _id:String;

    // pattern for the rule name
    private var _rulePattern:String = null;

    /**
     * Allowed levels
     * WARNING: these values must match those in the RemoteClass
     */
    public static var NaP:int = 0;
    public static var INFO:int = 1;
    public static var WARN:int = 2;
    public static var ERROR:int = 3;
    private var _level:int = NaP;

    /**
     * Allowed policies for levels
     * WARNING: these values must match those in the RemoteClass
     */
    public static var THIS:int = 0;
    public static var MIN:int = 1;
    private var _levelPolicy:int = THIS;

    /**
     * Details  [name] = pattern
     */
    private var _details:Dictionary = new Dictionary();
    
    public function NotifFilter():void {
    }


    public function get id():String {
        return _id;
    }

    public function set id(value:String):void {
        _id = value;
    }

    public function get rulePattern():String {
        return _rulePattern;
    }

    public function set rulePattern(value:String):void {
        _rulePattern = value;
    }

    public function get level():int {
        return _level;
    }

    public function set level(value:int):void {
        _level = value;
    }

    public function get levelPolicy():int {
        return _levelPolicy;
    }

    public function set levelPolicy(value:int):void {
        _levelPolicy = value;
    }

    /**
     * Set the level from the string in configuration file
     * @param str
     */
    public function setLevel(str:String):void {
        if (str == "Info") {
            _level = NotifFilter.INFO;
        } else if (str == "Warn") {
            _level = NotifFilter.WARN;
        }  else if (str == "Error") {
            _level = NotifFilter.ERROR;
        } else {
            _level = NotifFilter.NaP;
        }
    }

    /**
     * Set the level policy from the string in configuration file
     * @param str
     */
    public function setLevelPolicy(str:String):void {
        if (str == "Min") {
            _levelPolicy = MIN;
        } else {
            _levelPolicy = THIS;
        }
    }

    /**
     * Set a detail from configuration file
     * @param name
     * @param pattern
     */
    public function setDetail(name:String, pattern:String):void {
        _details[name] = pattern;
    }

    public function get details():Dictionary {
        return _details;
    }

    /**
     * @param notif JasmineEventNotificationVO to be checked against this filter
     * @return true if this notif matches this filter
     */
    public function matches(notif:JasmineEventNotificationVO):Boolean {
        // rule
        if (rulePattern != null) {
            var rm:Array = notif.rule.match(new RegExp(rulePattern));
            if (rm == null || rm.length == 0) {
                return false;
            }
        }
        // level
        if (level != NaP) {
            if (levelPolicy == THIS && levelFromString(notif.level) != level) {
                return false;
            }
            if (levelPolicy == MIN && levelFromString(notif.level) < level) {
                return false;
            }
        }
        // details
        for (var name:String in details) {
            var pattern:String = details[name];
            var res:String = notif.hasDetail(name);
            rm = res.match(new RegExp(pattern));
            if (rm == null || rm.length == 0) {
                return false;
            }
        }
        return true;
    }

    /**
     * Find the level, given the level String in a JasmineEventNotificationVO
     * @param str
     * @return int level
     */
    private function levelFromString(str:String):int {
        if (str.charAt(0) == 'E' || str.charAt(0) == 'e') {
            return ERROR;
        }       
        if (str.charAt(0) == 'W' || str.charAt(0) == 'w') {
            return WARN;
        }
        if (str.charAt(0) == 'I' || str.charAt(0) == 'i') {
            return INFO;
        }
        return NaP;
    }
}
}
